;>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
;Program demonstrate the use of a 64x128 OLED display connected via a SSD1306 controller
;>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
Device = 18F26K22                                   ;Set device

    ;Internal Oscillator 16Mhz and enable PLL (= 4 * 16 = 64Mhz)
OSCCONbits_IRCF2 = 1 : OSCCONbits_IRCF1 = 1 : OSCCONbits_IRCF0 = 1  ;16Mhz internal clock
OSCTUNEbits_PLLEN = 1                               ;Enable PLL 
DelayMS 50                                          ;After writing these registers the oscillator becomes unstable for a while
Declare Xtal = 64                                   ;16 x 4 = 64Mhz

Declare All_Digital = True                          ;Set all A\I to be D\I
TRISA = 0 : TRISB = 0 : TRISC = 0                   ;All output
Declare Hbus_Bitrate  1000                          ;Not needed. Also in the SSD1306_Def.inc                     
;-----------------------------------------------
;###############################################
;###############################################
;###############################################
;   VersionAdministration
Symbol  Version      = 0        
Symbol  VersionSub   = 60       
;###############################################
;###############################################
;###############################################
;-----------------------------------------------
    ;I2C protocol MSSP1 only, SPI protocol MSSP1 or MSSP2
    ;Define protocol: default is I2C $define SSD1306_Protocol_I2C
;$define SSD1306_Protocol_SPI            ;*****UnComment this $define for SPI use*****
$ifdef SSD1306_Protocol_SPI
;*****I use the xxxxxx_HSPI.Inc from AlbertoFS*****
;*****See for 18F2xKxx: http://www.protonbasic.co.uk/showthread.php?t=75171
;*****See for 18F4xKxx: http://www.protonbasic.co.uk/showthread.php?t=75212
        ;Define the MSSPx which will be used
        ;MSSP1 SPImaster-SKL1=RC3 SDI1=RC4 SDO1=RC5 (D0=SKL1 D1=SDO1) 
;    $define SSD1306_SPI1                ;*****comment if MSSP2 is used*****
        ;MSSP2 SPImaster-SKL2=RB1 SDI2=RB2 SDO2=RB3 (D0=SKL2 D1=SDO2) 
    $define SSD1306_SPI2                ;*****comment if MSSP1 used*****
        ;Define the 3 signal pins
    $define SSD1306_CS_Pin LATB.0       ;*****Change as needed*****
    $define SSD1306_RES_Pin LATB.4      ;*****Change as needed*****
    $define SSD1306_DC_Pin LATB.5       ;*****Change as needed*****
        ;Set all the needed TRIS
    Output SSD1306_CS_Pin
    Output SSD1306_RES_Pin
    Output SSD1306_DC_Pin
    $ifdef SSD1306_SPI1                 ;Set SPI1 pins if needed
        Output LATC.3
        Input PORTC.4                   ;we don't use input
        Output LATC.5
    $endif
    $ifdef SSD1306_SPI2                 ;Set SPI2 pins if needed
        Output LATB.1
        Input PORTB.2                   ;we don't use input
        Output LATB.3
    $endif
        ;Include de Amicus_HSPI from AlbertoFS
    Include "Amicus_HSPI.inc"
        ;Switch SSD1306 ChipSelect and Reset off (=High)
    High SSD1306_CS_Pin 
    High SSD1306_RES_Pin
$endif
    ;Define Display: Default is 64x128 $define SSD1306_DisplayNormal
;$define SSD1306_DisplayHalf                        ;*****UnComment if 32x128 display*****

    ;Include the SSD1306_Def and Graphic_Def
Include "SSD1306_Def_v1.10.inc"
Include "Graphic_Def_v0.4.inc"

Dim var1                    As Word                 ;Dim var
Dim loopCount               As Byte                 ;Dim var, general counter
Dim loopCount2              As Byte                 ;Dim var, general counter

Clear                                               ;Clear memory
    ;Include for the hardware the SSD1306_Sub and for the graphics Graphic_Sub
    ;No jump over needed, the files have their own
Include "SSD1306_Sub_v1.10.inc"      ;SSD1306_Init() automatic executed but it is allowed to do it again!!
Include "Graphic_Sub_v0.4.inc"   

;>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
; Main program - Test some fonts
;>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
While 1 > 0
    Cls                                             'Clear VideoRam and Refresh display
    
    DrawRectangle(0,0,127,63,1)                     'draw a rectangle on the display edge  (x0, y0, x1, y1, color)
    FillRectangle(3,3,124,14,1)                     'draw a filled rectangle  (x0, y0, x1, y1, Color)
    SelectFont(Arial_9)                             'select the font to be used for the next PRINT 
    Print Inverse 1                                 'invert the drawn pixel color
    Print Xor 1                                     'print transparent like
    Print At 12,3,"Arial 9p v",Dec1 Version, ".", Dec2 VersionSub 'print a string at x,y coords
    Print Xor 0                                     'do not print transparent like 
    Print Inverse 0                                 'do not print Inverse    
    
    SelectFont(Arial_6)                             'select the font to be used for the next PRINT 
    Print At 3,16, "This is Arial 6p"               'Print something at x,y coords
   
    SelectFont(Calibri_10)                          'select the font to be used for the next PRINT 
    Print At 3,24, "This is Calibri 10p"            'Print something at x,y coords

    SelectFont(Arial_16)                            'select the font to be used for the next PRINT 
    Print At 3,34, "And Arial 16p"                  'Print something at x,y coords
        
    GoSub PrintCount                                'A print gosub for the loop counter
           
    RefreshDisplay()                                'Send video buffer (PIC RAM) to the display  
    
    DelayMS 3000                                    'Delay for a while
    
;>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
; Draw some rectangles
;>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
    For loopCount = 0 To 2
            ;Cls has the same result as:
        ClearVideoMem()                             ;First: clear PIC RAM video buffer
        RefreshDisplay()                            ;Second: Refresh Display
                                                   
        DrawRectangle(0,0,127,63,1)                 'draw a rectagle on the display edge  (x0, y0, x1, y1, color)
        GoSub PrintCount                            'A print gosub for the loop counter   
        RefreshDisplay()                            ;Refresh Display
        
        DrawRectangle(5,5,122,58,1)                 'draw a rectagle
        GoSub PrintCount                            'A print gosub for the loop counter   
        RefreshDisplay()                            ;Refresh Display
        
        DrawRectangle(10,10,117,53,1)               'draw a rectagle
        GoSub PrintCount                            'A print gosub for the loop counter   
        RefreshDisplay()                            ;Refresh Display
        
        DrawRectangle(15,15,112,48,1)               'draw a rectagle
        GoSub PrintCount                            'A print gosub for the loop counter   
        RefreshDisplay()                            ;Refresh Display
        
        DrawRectangle(20,20,107,43,1)               'draw a rectagle
        GoSub PrintCount                            'A print gosub for the loop counter   
        RefreshDisplay()                            ;Refresh Display
        
        DrawRectangle(25,25,102,38,1)                'draw a rectagle
        GoSub PrintCount                            'A print gosub for the loop counter   
        RefreshDisplay()                            ;Refresh Display
        
    Next loopCount
    
;>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
; Draw some circles
;>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
    For loopCount = 0 To 2
        Cls                                         'clear screen
                                                   
        DrawCircle(64, 32, 5, 1)                    'Draw a circle Xm, Ym, RADIUS, color                 
        GoSub PrintCount                            'A print gosub for the loop counter   
        RefreshDisplay()                            'Send video buffer (PIC RAM) to the display  

        DrawCircle(64, 32, 10, 1)                   'Draw a circle
        GoSub PrintCount                            'A print gosub for the loop counter   
        RefreshDisplay()                            'Send video buffer (PIC RAM) to the display  

        DrawCircle( 64, 32, 15, 1)                  'Draw a circle
        GoSub PrintCount                            'A print gosub for the loop counter   
        RefreshDisplay()                            'Send video buffer (PIC RAM) to the display  

        DrawCircle( 64, 32, 20, 1)                  'Draw a circle
        GoSub PrintCount                            'A print gosub for the loop counter   
        RefreshDisplay()                            'Send video buffer (PIC RAM) to the display 

        DrawCircle( 64, 32, 25, 1)                  'Draw a circle
        GoSub PrintCount                            'A print gosub for the loop counter   
        RefreshDisplay()                            'Send video buffer (PIC RAM) to the display 

        DrawCircle( 64, 32, 30, 1)                  'Draw a circle
        GoSub PrintCount                            'A print gosub for the loop counter   
        RefreshDisplay()                            'Send video buffer (PIC RAM) to the display 

        DrawCircle( 64, 32, 35, 1)                  'Draw a circle
        GoSub PrintCount                            'A print gosub for the loop counter   
        RefreshDisplay()                            'Send video buffer (PIC RAM) to the display 

        DrawCircle( 64, 32, 40, 1)                  'Draw a circle
        GoSub PrintCount                            'A print gosub for the loop counter   
        RefreshDisplay()                            'Send video buffer (PIC RAM) to the display   

        DrawCircle(64, 32, 45, 1)                   'Draw a circle
        GoSub PrintCount                            'A print gosub for the loop counter   
        RefreshDisplay()                            'Send video buffer (PIC RAM) to the display   

        DrawCircle(64, 32, 50, 1)                   'Draw a circle
        GoSub PrintCount                            'A print gosub for the loop counter   
        RefreshDisplay()                            'Send video buffer (PIC RAM) to the display   

        DrawCircle(64, 32, 55, 1)                   'Draw a circle
        GoSub PrintCount                            'A print gosub for the loop counter   
        RefreshDisplay()                            'Send video buffer (PIC RAM) to the display    
        
    Next loopCount
    
;>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
; Draw some lines
;>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
    For loopCount = 0 To 2
        Cls                                         ;ClearVideoMem() and RefreshDisplay()
        DrawLine(0,0,127,5,1)                       ;Draw line from left>right
        GoSub PrintCount                            'print loop counter
        RefreshDisplay()

        DrawLine(127,10,0,0,1)                      ;Draw line from lright>left
        GoSub PrintCount                            'print loop counter
        RefreshDisplay()

        DrawLinTo(127,15,1)                         ;Draw line from last endpoint>right
        GoSub PrintCount                            'print loop counter
        RefreshDisplay()

        DrawLine(127,20,0,0,1)                      ;Draw line from lright>left
        GoSub PrintCount                            'print loop counter
        RefreshDisplay()

        DrawLinTo(127,25,1)                         ;Draw line from last endpoint>right
        GoSub PrintCount                            'print loop counter
        RefreshDisplay()

        DrawLine(127,30,0,0,1)                      ;Draw line from lright>left
        GoSub PrintCount                            'print loop counter
        RefreshDisplay()

        DrawLinTo(127,35,1)                         ;Draw line from last endpoint>right
        GoSub PrintCount                            'print loop counter
        RefreshDisplay()

        DrawLine(127,40,0,0,1)                      ;Draw line from lright>left
        GoSub PrintCount                            'print loop counter
        RefreshDisplay()

        DrawLinTo(127,45,1)                         ;Draw line from last endpoint>right
        GoSub PrintCount                            'print loop counter
        RefreshDisplay()

        DrawLine(127,50,0,0,1)                      ;Draw line from lright>left
        GoSub PrintCount                            'print loop counter
        RefreshDisplay()

        DrawLinTo(127,55,1)                         ;Draw line from last endpoint>right
        GoSub PrintCount                            'print loop counter
        RefreshDisplay()

        DrawLine(127,60,0,0,1)                      ;Draw line from lright>left
        GoSub PrintCount                            'print loop counter
        RefreshDisplay()

        DrawLinTo(127,63,1)                         ;Draw line from last endpoint>right
        GoSub PrintCount                            'print loop counter
        RefreshDisplay()

        For loopCount2 = 125 To 0 Step -5    
            DrawLine(0,0,loopCount2,63,1)           ;draw lines from 0.0 to bottom
            RefreshDisplay()
        Next
    Next

    Cls
;>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
;Some Inversed  stuff
;>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
    FillRectangle(0,0,127,63,1)                     'draw a filled rectangle  (x0, y0, x1, y1, Color)

    SelectFont(Arial_16)                            'select the font to be used for the next PRINT 
    Print Inverse 1                                 'invert the drawn pixel color
    Print Xor 1                                     'print transparent like
    Print At 25,20,"**THE**"                        'print a string at absolute display coordinates
    Print At 25,37,"**END**"                        'print a string at absolute display coordinates
    Print Xor 0                                     'do not print transparent like 
    Print Inverse 0                                 'do not print Inverse
        
    RefreshDisplay()                                'Send video buffer (PIC RAM) to the display (GDDRAM)
    DelayMS 2000                            
;>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
;Use some direct SSD1306 commands
;>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
    SSD1306_Contrast(0)                             ;Grey display with visible pixels
    DelayMS 2000
    SSD1306_Contrast(255)                           ;Default INIT value in the Library
    DelayMS 2000

    SSD1306_DisplayOff()                            ;Switch Display Off (sleep mode)
    DelayMS 1000
    SSD1306_DisplayOn()                             ;Switch Display On (normal mode)

    SSD1306_DisplayInverse()                        ;Inverse Display at once - NO CHANGE in the GDDRAM or PIC RAM
    DelayMS 1000
    SSD1306_DisplayNormal()                         ;Normal Display at once - NO CHANGE in the GDDRAM or PIC RAM
    DelayMS 1000
        ;Now **THE** and **END** are still in inverse in the PIC RAM, nothing has changed in PIC RAM!!!!
    SSD1306_FlipDisplay()                            ;Flip Display at once - NO CHANGE in the GDDRAM or PIC RAM
    DelayMS 1000
    SSD1306_FlipDisplay()                            ;Flip Display at once back - NO CHANGE in the GDDRAM or PIC RAM
    DelayMS 2000
                                
                                
;>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
;Draw Black box of 10x10 pixels on the inverse background
;drawPixel was already available but not in the Demo.bas
;>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
    For loopCount = 15 To 24
        For loopCount2 = 45 To 54
            DrawPixel(loopCount, loopCount2, 0)
        Next
    Next
    RefreshDisplay()                                
    DelayMS 2000
    
    ClearVideoMem()                                 ;Clear video buffer (PIC RAM)
    SelectFont(ArialNarrow_36_NumOnly)              ;Use the NumOnly Fontset 
    Print At 10,15, "23:59"
    RefreshDisplay()                                ;Send video buffer (PIC RAM) To the display (GDDRAM)
    DelayMS 2000

Wend                                                'Loop to the start 
;>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
PrintCount:
    SelectFont(Arial_9)                             'select the font to be used for the next PRINT 
    Print At 3,51, "Count: ", Dec var1, " "         'Print the loop counter
    Inc var1                                        'increment var
    DelayMS 100
Return
End    

;>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
; include FONTS section (result of Proton FontConverter.exe
;>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
Include "Arial_6-9x8.inc"
Include "Arial_9-12x13.inc"
Include "Arial_16-23x21.inc"
Include "Calibri_10-13x11.inc"
Include "ArialNarrow_36-42x47_NumOnly.inc"
;===========================================================
;=============F U S E S=====================================
;===========================================================
Include "18F26K2x_Fuses_IntOSC.inc"

