$ifndef _SSD1306_Def_
$define _SSD1306_Def_
'***************************************************************************
'***************************************************************************
'***************************************************************************
'***************************************************************************
'***************************************************************************
'*  Name     : SSD1306_Def_v0.x.inc                                              
'*  Founding Fathers : Gabi, VaGyver, Ray Ellam, Les Johnson                
'*  Version  : 0.2-6th September 2015                                                          
'*  Version  : 0.3-24th November 2016                                                          
'*  Version  : 0.4-13th September 2017                                      
'*           :     Two parts: 
'*           :     1 - SSD1306_Def and SSD1306_Sub for the SSD1306 Control chip                                                        
'*           :     2 - Graphic_Def and Graphic_Sub for the graphic routines                                                        
'*           :     Display 32x128 added
'*           :     Some errors and problems and unused code removed
'*           :     Calling method changed via pre processor                                                         
'*           :     Not Used routines will not be compiled                                                         
'*  Version  : 1.00-20171017 SSD1306_FlipDisplay() added
'*           :     SSD1306_Init() clears graphic RAM before panel switch On                                                            
'*           :     Not used variables deleted                                                         
'*  Version  : 1.10-20171217 SPI protocol added. Macro's Amicus18_HSPI (from AlbertoFS) used
'*           :      See: http://www.protonbasic.co.uk/showthread.php?t=75171                                                        
'*           :                                                              
'*           :                                                              
'***************************************************************************
;
;   DEFINITIONS
;
;***************************************************************************
;-----------------------------------------------
;###############################################
;###############################################
;###############################################
;   VersionAdministration
Symbol  SSD1306_Def_Version      = 1        
Symbol  SSD1306_Def_VersionSub   = 10       
;###############################################
;###############################################
;###############################################
;***************************************************************************
;Default protocol I2C alternative $Define SSD1306_Protocol_SPI
$ifndef SSD1306_Protocol_SPI
    $define SSD1306_Protocol_I2C
$endif
; Default is 128x64 SSD1306_DisplayNormal. Possible $define SSD1306_DisplayHalf                               
$ifndef SSD1306_DisplayHalf
    $define SSD1306_DisplayNormal       ;Default value
$endif
;---------------------------------------------------------------------------
;   Create System definitions
;---------------------------------------------------------------------------
$ifdef SSD1306_Protocol_I2C
    Symbol  SSD1306         = $3C << 1          ;I2C address SSD1306 (=$3C) Bit0=R/W=1/0
$endif
$ifdef SSD1306_DisplayNormal
    Symbol  DisplayWidth    = 128           ;# pixels
    Symbol  DisplayHeight   = 64            ;# pixels
    Symbol  ArraySize = ( DisplayWidth * DisplayHeight ) / 8       ;# pixels / 8 = # bytes    
$endif
$ifdef SSD1306_DisplayHalf
        ;32x128 display uses the whole SSD1306 GDRAM of 1024bytes Only uneven Y-bits will be displayed
        ;So for the SSD1306 it is a 128x64 display. 
    Symbol  DisplayWidth    = 128           ;# pixels
    Symbol  DisplayHeight   = 64            ;# pixels              ;only 4bits/byte
    Symbol  ArraySize = ( DisplayWidth * DisplayHeight ) / 8       ;# pixels / 8= # bytes            
$endif
#disable GLCD_CLS                           ;Clear Screen Proton disabled
;---------------------------------------------------------------------------
;   SSD1306 declares
$ifdef SSD1306_Protocol_I2C
            Declare Warnings Off                ;Very ugly BUT: No Warning if the user has already used the declare
    Declare Hbus_Bitrate    =   1000            ;Works on 1000 instead 400
            Declare Warnings On
            ;See page 20 of the datasheet SSD1306
    Symbol  ControlByteCMD  = %10000000         ;Co bit set = Command follows
    Symbol  ControlByteData = %01000000         ;D/C# bit set = GDRAM byte follows
$endif
$ifdef SSD1306_Protocol_SPI
    Symbol  DCcommand       = 0                 ;D/C# line Low for CommandMode
    Symbol  DCdata          = 1                 ;D/C# line High for DataMode
    Symbol  CSactiv         = 0                 ;ChipSelect Low is activ
    Symbol  CSidle          = 1                 ;ChipSelect High is idle
$endif
;---------------------------------------------------------------------------
;   Create variables
;---------------------------------------------------------------------------
;   Videobuffer in PIC memory
;   DISPLAY=128 cols x 64 rows= 8192bits= 1024Bytes
;   SSD1306 GDRAM= 128 cols x 8bit(=1Byte)= Page Total 128 cols x 8 pages
;   SSD1306 Page= 128 bytes = 128*8=1024pixels
;   SSD1306 Minimum Write = 1 Byte = 8 pixels. So there is no "real pixel" cursor position 
Dim     bVideoMem[ArraySize] As Byte    ;videobuffer 128x64 pixels=bits  
Dim     wElement#No  As Word            ;Array element in VideoMem
Dim     FSR0        As FSR0L.Word       ;Addressing of variable
;------------------------------------------------------------------------------
;   SSD1306
Dim     SSD1306_bCommand    As Byte     ;Command byte
Dim     SSD1306_bContrast   As Byte     ;Contrast value
Dim     SSD1306_xFlipped    As Byte     ;Is display Flipped Yes=1 No=0
;-------------------------------------------------------------------------------
;   SSD1306_Plot
Dim     SSD1306_bPlotX      As Byte     ;SSD1306_Plot X
Dim     SSD1306_bPlotY      As Byte     ;SSD1306_Plot Y
Dim     SSD1306_xPlotColor  As Bit      ;SSD1306_Plot Color 1=White 0=Black
Dim     SSD1306_bPlotByte   As Byte     ;SSD1306_Plot Byte uit VideoMem
Dim     SSD1306_bPlotBitNum As Byte     ;SSD1306_Plot Bit positie in VideoMem Byte
;-------------------------------------------------------------------------------
;   Temp variable
Dim     SSD1306_wTemp       As Word
;***************************************************************************
;   END SSD1306_Def.Inc
;***************************************************************************
$endif
;===========================================================================

